@@ -4,7 +4,7 @@ __all__ = [
     "DisposableConnection",
     "InParam",
     "OutParam",
-    "Param",
+    "TransactionManager",
     "check",
     "execute_non_query",
     "get_data",
@@ -15,87 +15,17 @@ __all__ = [
 ]
 
 from types import TracebackType
-from typing import Any, List, Optional, Tuple, Type, Union
+from typing import Any, List, Optional, Tuple, Type, TypedDict, Union
 
 import system.db
+import system.util
 from com.inductiveautomation.ignition.common import BasicDataset
 from java.lang import Thread
 
-from incendium.helper.types import AnyStr, DictIntStringAny, SProcResult
+from incendium.helper.types import AnyStr, DictIntStringAny
 
 
-class DisposableConnection(object):
-    """Disposable Connection.
-
-    A disposable connection enables a database connection in Ignition
-    and disables it once the operation is completed to release
-    resources.
-    """
-
-    database = None  # type: AnyStr
-    retries = None  # type: int
-
-    def __init__(self, database, retries=3):
-        # type: (AnyStr, int) -> None
-        """Disposable Connection initializer.
-
-        Args:
-            database: The name of the database connection in
-                Ignition.
-            retries: The number of additional times to retry
-                enabling the connection. Optional.
-        """
-        super(DisposableConnection, self).__init__()
-        self.database = database
-        self.retries = retries
-
-    def __enter__(self):
-        # type: () -> DisposableConnection
-        """Enter the runtime context related to this object.
-
-        Raises:
-            IOError: If the connection's status reports as Faulted, or
-                ir cannot be enabled.
-        """
-        system.db.setDatasourceEnabled(self.database, True)
-
-        for _ in range(self.retries):
-            Thread.sleep(1000)
-            if self.status == "Valid":
-                break
-            if self.status == "Faulted":
-                raise IOError(
-                    "The database connection {!r} is {}.".format(
-                        self.database, self.status
-                    )
-                )
-        else:
-            raise IOError(
-                "The database connection {!r} could not be enabled.".format(
-                    self.database
-                )
-            )
-        return self
-
-    def __exit__(
-        self,
-        exc_type,  # type: Optional[Type[BaseException]]
-        exc_val,  # type: Optional[BaseException]
-        exc_tb,  # type: Optional[TracebackType]
-    ):
-        # type: (...) -> None
-        """Exit the runtime context related to this object."""
-        system.db.setDatasourceEnabled(self.database, False)
-
-    @property
-    def status(self):
-        # type: () -> AnyStr
-        """Get connection status."""
-        connection_info = system.db.getConnectionInfo(self.database)
-        return str(connection_info.getValueAt(0, "Status"))
-
-
-class Param(object):
+class _Param(object):
     """Base class used for defining [IN|OUT]PUT parameters."""
 
     def __init__(
@@ -112,24 +42,11 @@ class Param(object):
             type_code: Type code constant.
             value: Value of type type_code.
         """
-        super(Param, self).__init__()
+        super(_Param, self).__init__()
         self._name_or_index = name_or_index
         self._type_code = type_code
         self._value = value
 
-    def __repr__(self):  # type: ignore[no-untyped-def]
-        """Compute the "official" string representation."""
-        return "{}(name_or_index={!r}, type_code={}, value={})".format(
-            self.__class__.__name__,
-            self.name_or_index,
-            self.type_code,
-            self.value,
-        )
-
-    def __str__(self):  # type: ignore[no-untyped-def]
-        """Compute the "informal" string representation."""
-        return "{!r}, {}, {}".format(self.name_or_index, self.type_code, self.value)
-
     @property
     def name_or_index(self):
         # type: () -> Union[int, AnyStr]
@@ -148,8 +65,114 @@ class Param(object):
         """Get value of value."""
         return self._value
 
+    def __repr__(self):  # type: ignore[no-untyped-def]
+        """Compute the "official" string representation."""
+        return "{}(name_or_index={!r}, type_code={}, value={})".format(
+            self.__class__.__name__,
+            self.name_or_index,
+            self.type_code,
+            self.value,
+        )
 
-class InParam(Param):
+    def __str__(self):  # type: ignore[no-untyped-def]
+        """Compute the "informal" string representation."""
+        return "{!r}, {}, {}".format(self.name_or_index, self.type_code, self.value)
+
+
+_SProcResult = TypedDict(
+    "_SProcResult",
+    {
+        "output_params": DictIntStringAny,
+        "result_set": BasicDataset,
+        "return_value": Optional[int],
+        "update_count": int,
+    },
+)
+
+
+class DisposableConnection(object):
+    """Disposable Connection.
+
+    A disposable connection enables a database connection in Ignition
+    and disables it once the operation is completed to release
+    resources.
+    """
+
+    def __init__(self, database, retries=3):
+        # type: (AnyStr, int) -> None
+        """Disposable Connection initializer.
+
+        Args:
+            database: The name of the database connection in Ignition.
+            retries: The number of additional times to retry enabling
+                the connection. Optional.
+        """
+        super(DisposableConnection, self).__init__()
+        self._database = database
+        self._retries = retries
+        self._global_conn = "incendium_db_{}".format(database)
+
+    @property
+    def database(self):
+        # type: () -> AnyStr
+        """Get the name of the disposable connection."""
+        return self._database
+
+    @property
+    def status(self):
+        # type: () -> AnyStr
+        """Get connection status."""
+        connection_info = system.db.getConnectionInfo(self._database)
+        return str(connection_info.getValueAt(0, "Status"))
+
+    def __enter__(self):
+        # type: () -> DisposableConnection
+        """Enter the runtime context related to this object.
+
+        Raises:
+            IOError: If the connection's status reports as Faulted, or
+                it cannot be enabled.
+
+        Returns:
+            DisposableConnection: The current instance.
+        """
+        system.db.setDatasourceEnabled(self._database, True)
+
+        for _ in range(self._retries):
+            Thread.sleep(1000)
+            if self.status == "Valid":
+                if self._global_conn not in system.util.globals:
+                    system.util.globals[self._global_conn] = 0
+                system.util.globals[self._global_conn] += 1
+                break
+            if self.status == "Faulted":
+                raise IOError(
+                    "The database connection {!r} is {}.".format(
+                        self._database, self.status
+                    )
+                )
+        else:
+            raise IOError(
+                "The database connection {!r} could not be enabled.".format(
+                    self._database
+                )
+            )
+        return self
+
+    def __exit__(
+        self,
+        exc_type,  # type: Optional[Type[BaseException]]
+        exc_val,  # type: Optional[BaseException]
+        exc_tb,  # type: Optional[TracebackType]
+    ):
+        # type: (...) -> None
+        """Exit the runtime context related to this object."""
+        system.util.globals[self._global_conn] -= 1
+        if system.util.globals[self._global_conn] == 0:
+            system.db.setDatasourceEnabled(self._database, False)
+
+
+class InParam(_Param):
     """Class used for declaring INPUT parameters."""
 
     def __init__(self, name_or_index, type_code, value):
@@ -157,15 +180,15 @@ class InParam(Param):
         """Create an instance of InParam.
 
         Args:
-            name_or_index: Index (int starting at 1, not 0), or
-                name (str).
+            name_or_index: Index (int starting at 1, not 0), or name
+                (str).
             type_code: Type code constant from `system.db`.
             value: Value of type type_code.
         """
         super(InParam, self).__init__(name_or_index, type_code, value)
 
 
-class OutParam(Param):
+class OutParam(_Param):
     """Class used for declaring OUTPUT parameters."""
 
     def __init__(self, name_or_index, type_code):
@@ -180,10 +203,58 @@ class OutParam(Param):
         super(OutParam, self).__init__(name_or_index, type_code)
 
 
+class TransactionManager(object):
+    """Database transaction context manager."""
+
+    def __init__(self, database="", isolation_level=None, timeout=None):
+        # type: (Optional[str], Optional[int], Optional[int]) -> None
+        """Transaction Manager initializer.
+
+        Args:
+            database: The name of the database connection to use. If not
+                specified, the project's default database is used.
+                Optional.
+            isolation_level: The isolation level for the transaction. If
+                not specified, the default isolation level is used.
+                Optional.
+            timeout: The timeout for the transaction in seconds. If not
+                specified, the default timeout is used. Optional.
+        """
+        self._database = database
+        self._isolation_level = isolation_level
+        self._timeout = timeout
+        self.transaction_id = ""  # type: AnyStr
+
+    def __enter__(self):
+        # type: () -> TransactionManager
+        """Enter the runtime context related to this object."""
+        self.transaction_id = system.db.beginTransaction(
+            database=self._database,
+            isolationLevel=self._isolation_level,
+            timeout=self._timeout,
+        )
+        return self
+
+    def __exit__(
+        self,
+        exc_type,  # type: Optional[Type[BaseException]]
+        exc_val,  # type: Optional[BaseException]
+        exc_tb,  # type: Optional[TracebackType]
+    ):
+        # type: (...) -> None
+        """Exit the runtime context related to this object."""
+        if exc_type is None:
+            system.db.commitTransaction(self.transaction_id)
+        else:
+            system.db.rollbackTransaction(self.transaction_id)
+
+        system.db.closeTransaction(self.transaction_id)
+
+
 def _execute_sp(
     stored_procedure,  # type: AnyStr
     database="",  # type: AnyStr
-    transaction=None,  # type: Optional[AnyStr]
+    transaction_id=None,  # type: Optional[AnyStr]
     skip_audit=False,  # type: bool
     in_params=None,  # type: Optional[List[InParam]]
     out_params=None,  # type: Optional[List[OutParam]]
@@ -193,7 +264,7 @@ def _execute_sp(
     get_update_count=False,  # type: bool
     return_type_code=system.db.INTEGER,  # type: int
 ):
-    # type: (...) -> SProcResult
+    # type: (...) -> _SProcResult
     """Execute a stored procedure against the connection.
 
     Args:
@@ -201,8 +272,8 @@ def _execute_sp(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        transaction: A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         skip_audit: A flag which, if set to True, will cause the
             procedure call to skip the audit system. Useful for some
             queries that have fields which won't fit into the audit log.
@@ -227,7 +298,7 @@ def _execute_sp(
     call = system.db.createSProcCall(
         procedureName=stored_procedure,
         database=database,
-        tx=transaction,
+        tx=transaction_id,
         skipAudit=skip_audit,
     )
 
@@ -261,8 +332,13 @@ def _execute_sp(
     }
 
 
-def check(stored_procedure, database="", params=None):
-    # type: (AnyStr, AnyStr, Optional[List[InParam]]) -> Optional[bool]
+def check(
+    stored_procedure,  # type: AnyStr
+    database="",  # type: AnyStr
+    transaction_id=None,  # type: Optional[AnyStr]
+    params=None,  # type: Optional[List[InParam]]
+):
+    # type: (...) -> Optional[bool]
     """Execute a stored procedure against the connection.
 
     This will return a flag set to TRUE or FALSE.
@@ -272,23 +348,33 @@ def check(stored_procedure, database="", params=None):
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         params: A Dictionary containing all INPUT parameters. Optional.
 
     Returns:
         The flag.
     """
-    output = OutParam("flag", system.db.BIT)
-    output_params = get_output_params(
-        stored_procedure, output=[output], database=database, params=params
+    result = _execute_sp(
+        stored_procedure,
+        database=database,
+        transaction_id=transaction_id,
+        in_params=params,
+        out_params=[OutParam("flag", system.db.BIT)],
+        get_out_params=True,
     )
 
-    return output_params["flag"] if "flag" in output_params.iterkeys() else None
+    return (
+        result["output_params"]["flag"]
+        if "flag" in result["output_params"].iterkeys()
+        else None
+    )
 
 
 def execute_non_query(
     stored_procedure,  # type: AnyStr
     database="",  # type: AnyStr
-    transaction=None,  # type: Optional[AnyStr]
+    transaction_id=None,  # type: Optional[AnyStr]
     params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> int
@@ -301,8 +387,8 @@ def execute_non_query(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        transaction: A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -313,7 +399,7 @@ def execute_non_query(
     result = _execute_sp(
         stored_procedure,
         database=database,
-        transaction=transaction,
+        transaction_id=transaction_id,
         in_params=params,
         get_update_count=True,
     )
@@ -324,6 +410,7 @@ def execute_non_query(
 def get_data(
     stored_procedure,  # type: AnyStr
     database="",  # type: AnyStr
+    transaction_id=None,  # type: Optional[AnyStr]
     params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> BasicDataset
@@ -334,6 +421,8 @@ def get_data(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -344,6 +433,7 @@ def get_data(
     result = _execute_sp(
         stored_procedure,
         database=database,
+        transaction_id=transaction_id,
         in_params=params,
         get_result_set=True,
     )
@@ -355,7 +445,7 @@ def get_output_params(
     stored_procedure,  # type: AnyStr
     output,  # type: List[OutParam]
     database="",  # type: AnyStr
-    transaction=None,  # type: Optional[AnyStr]
+    transaction_id=None,  # type: Optional[AnyStr]
     params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> DictIntStringAny
@@ -368,8 +458,8 @@ def get_output_params(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        transaction: A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -379,7 +469,7 @@ def get_output_params(
     result = _execute_sp(
         stored_procedure,
         database=database,
-        transaction=transaction,
+        transaction_id=transaction_id,
         in_params=params,
         out_params=output,
         get_out_params=True,
@@ -392,7 +482,7 @@ def get_return_value(
     stored_procedure,  # type: AnyStr
     return_type_code,  # type: int
     database="",  # type: AnyStr
-    transaction=None,  # type: Optional[AnyStr]
+    transaction_id=None,  # type: Optional[AnyStr]
     params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> Optional[int]
@@ -404,8 +494,8 @@ def get_return_value(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        transaction: A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -415,7 +505,7 @@ def get_return_value(
     result = _execute_sp(
         stored_procedure,
         database=database,
-        transaction=transaction,
+        transaction_id=transaction_id,
         in_params=params,
         get_ret_val=True,
         return_type_code=return_type_code,
@@ -428,7 +518,7 @@ def o_execute_non_query(
     stored_procedure,  # type: AnyStr
     out_params,  # type: List[OutParam]
     database="",  # type: AnyStr
-    transaction=None,  # type: Optional[AnyStr]
+    transaction_id=None,  # type: Optional[AnyStr]
     in_params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> Tuple[int, DictIntStringAny]
@@ -444,8 +534,8 @@ def o_execute_non_query(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        transaction: A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         in_params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -456,7 +546,7 @@ def o_execute_non_query(
     result = _execute_sp(
         stored_procedure,
         database=database,
-        transaction=transaction,
+        transaction_id=transaction_id,
         in_params=in_params,
         out_params=out_params,
         get_out_params=True,
@@ -470,6 +560,7 @@ def o_get_data(
     stored_procedure,  # type: AnyStr
     out_params,  # type: List[OutParam]
     database="",  # type: AnyStr
+    transaction_id=None,  # type: Optional[AnyStr]
     in_params=None,  # type: Optional[List[InParam]]
 ):
     # type: (...) -> Tuple[BasicDataset, DictIntStringAny]
@@ -482,6 +573,8 @@ def o_get_data(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
+        transaction_id: A transaction identifier. If omitted, the call
+            will be executed in its own transaction. Optional.
         in_params: A list containing all INPUT parameters as InParam
             objects. Optional.
 
@@ -492,6 +585,7 @@ def o_get_data(
     result = _execute_sp(
         stored_procedure,
         database=database,
+        transaction_id=transaction_id,
         in_params=in_params,
         out_params=out_params,
         get_out_params=True,
